<?
 /*
  * I10 PHP connection class
  *
  * (c) 2002 by Me@lourensrozema.nl
  *
  * http://www.lourensrozema.nl/?page=i10.inc
  *
  */

 class i10 {
   var $max_packet_size = 128;

   var $handlers;
   var $socket;

   var $timeout;
   var $host;
   var $user;
   var $user;
   var $pass;

   var $errno;
   var $errstr;

   var $packet;
   var $io;

   var $debug = false;

   /*
    * I10 handler initializer
    */
   function i10($host, $port = 1100, $user = "admin", $pass, $timeout = 5) {
     // Store timeout, host, port, user and pass
     $this->timeout = $timeout;
     $this->host = $host;
     $this->port = $port;
     $this->user = $user;
     $this->pass = $pass;
   }

   /*
    * I10 connection method
    */
   function connect() {
     // Open connection
     $socket = fsockopen($this->host, $this->port, $errno, $errstr, $this->timeout);

     // Check connection
     if(! $socket) {
       // Return error
       $this->errno  = $errno;
       $this->errstr = $errstr;

       return(-1);
     }

     // Store socket
     $this->socket = $socket;

     // Return socket
     return(0);
   }

   /*
    * I10 close method
    */
   function close() {
     // Close socket
     fclose($this->socket);

     return(0);
   }

   /*
    * I10 control methods
    */

   function tick() {
     // Reset
     $this->errno = 0;
     $packet = 0;
     $io = 0;

     // Open connection
     if($this->connect() < 0) return(-1);

     // Drive the connection
     while(1) if($this->mrx() < 0) break;

     // Connection closed by foreign host?
     if($this->errno == 10002) return(0);

     // Force connection close
     $this->close();

     // Nothing wrong?
     if($this->errno == 0) return(0);

     return(-1);
   }

   function mtx_set($message, $code, $sub) {
     // Store packet
     $this->packet[message] = $message;
     $this->packet[code]    = $code;
     $this->packet[sub]     = $sub;

     // Drive a connection
     $r = $this->tick();

     return($r);
   }

   function set_output($port, $state) {
     // Store message
     $r = $this->mtx_set(12, $port, $state);

     return($r);
   }

   function read_io() {
     // Fetch list
     $r = $this->tick();

     return($r);
   }

   /*
    * I10 internal methods
    */

   function mrx_packet_system($code, $sub, $argc, $argv) {
     switch($code) {
       case(1):
         // Authenticate ourselfs
         $this->mtx(6, 2, 0, Array($this->user, $this->pass));
         break;
       case(2):
         // Check authentication
         if($sub == 255) {
           // Return error
           $this->errstr = "Authentication failed.";
           $this->errno  = 10001;

           return(-1);
         }

         // Take action
         if(is_array($this->packet)) {
           $this->mtx($this->packet[message], $this->packet[code], $this->packet[sub], NULL);
         } else {
           $this->mtx(1, 3, 1, NULL);
           $this->mtx(1, 4, 1, NULL);
         }

         // Logout
         $this->mtx(6, 3, 0, NULL);

         break;
       case(3):
         // Close connection
         $this->close();

         // Report
         $this->errstr = "Connection closed by foreign host.";
         $this->errno  = 10002;

         return(-1);
     }

     return;
   }

   function mrx_packet($message, $code, $sub, $argc, $argv) {
     $r = 0;

     switch($message) {
       case(6):		// System packet
         $r = $this->mrx_packet_system($code, $sub, $argc, $argv);
         break;
       case(10):	// Config IO
         $this->io[$code][direction] = ($sub == 1);
         break;
       case(11):	// List input
       case(12):	// List output
         $this->io[$code][state] = ($sub == 1);
         break;
     }

     return($r);
   }

   function mrx_debug($message, $code, $sub, $argc, $argv) {
     // Show packet info
     printf("Packet: message=$message, code=$code, sub=$sub, argc=$argc;\n");
  
     // Show arguments
     for($i = 0; $i < $argc; $i++) printf(" + argv[$i]=\"$argv[$i]\"\n");

     return;
   }

   function read_length() {
     // Receive length
     $buffer = fread($this->socket, 2);

     // Check buffer
     if(empty($buffer) or strlen($buffer) != 2) return(-1);

     // Calculate length
     $l = ord($buffer[1]) * 255 + ord($buffer[0]);

     return($l);
   }

   function mrx() {
     $r = 0;

     // Receive packet length
     if(($length = $this->read_length()) < 0) {
       // Return error
       return(-1);
     }

     // Receive packet
     $buffer = fread($this->socket, $length);

     // Check buffer
     if((! empty($buffer)) or strlen($buffer) == $length) {
       // Fetch message, code and sub
       $message = ord($buffer[0]);
       $code    = ord($buffer[1]);
       $sub     = ord($buffer[2]);

       // Fetch argument count
       $argc = ord($buffer[3]);

       // Set pointer
       $p = 4;

       // Fetch arguments
       for($i = 0; $i < $argc; $i++) {
         // Calculate argument length
         $l  = ord($buffer[$p + 1]) * 255 + ord($buffer[$p]);
         $p += 2;

         if($p > $length) break;

         // Store argument
         $argv[$i] = substr($buffer, $p, $l);
         $p += $l;
       }

       // Show the packet
       if($this->debug) $this->mrx_debug($message, $code, $sub, $argc, $argv);

       // Process the receive packet
       $r = $this->mrx_packet($message, $code, $sub, $argc, $argv);
     }

     // Wait until we've received a terminating null byte
     while(1) {
       // Receive a byte
       $dummy = fread($this->socket, 1);

       // Check byte
       if(empty($dummy) or strlen($dummy) != 1) return(-1);

       // Check for terminating null byte
       if(ord($dummy) == 0) break;
     }

     return($r);
   }

   function mtx($message, $code, $sub, $data = "") {
     // Set message code
     $buffer[2] = $message;
     $buffer[3] = $code;
     $buffer[4] = $sub;

     $p = 6;

     // Fill buffer with data
     if(is_array($data))
       for($i = 0; $i < count($data); $i++) {
         // Check buffer size
         if($p >= $this->max_packet_size) break;

         // Store argument length
         $buffer[$p++] = strlen($data[$i]) & 255;
         $buffer[$p++] = strlen($data[$i]) >> 8;

         // Copy data into buffer
         for($j = 0; $j < strlen($data[$i]); $j++)
           $buffer[$p++] = ord(substr($data[$i], $j, 1));
       }

     // Check buffer size
     if($p < $this->max_packet_size) {
       // Close packet
       $buffer[$p] = 0;

       // Set packet size
       $buffer[0] = ($p - 2) & 255;
       $buffer[1] = ($p - 2) >> 8;

       // Set argument count
       $buffer[5] = $i;

       // Convert buffer
       for($j = 0; $j < count($buffer); $j++)
         fputs($this->socket, chr($buffer[$j]), 1);
     }

     return;
   }
 }
?>
